% clear all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ChipRate = 3.84e6;
Pn_BS = 3.84e-14; %background noise power, NF=4dB
Pn_UE = 7.66e-14;
ALPHA = 0.4;%NF=7dB


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Pre-allocate Memory Space
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%downlink
MAX_BS_txpower = 20; %43dBm=20W

ActiveSet_ALL(1:MAX_NUM_UE,1:4,1:END_TIME) = 0; %128m
BS_Busy(1:MAX_NUM_ALL,1:END_TIME)= 0;
Number_Handover(1:MAX_NUM_UE) = 0;
UE_Cost_Handover(1:MAX_NUM_UE) = 0;

BS_HSDPA_txpower (1:MAX_NUM_BS) = 0.5 * MAX_BS_txpower ; %each BS's HSDPA power, 10W or zero

BS_Pilot_txpower(1:MAX_NUM_BS) = 1; %pilot channel power 30dBm=1W
BS_Common_txpower(1:MAX_NUM_BS) = 1; %combined power of other common channels 30dBm=1W

UE_total_rxpower(1:MAX_NUM_UE) = 0;
UE_total_rxpower_AP(1:MAX_NUM_UE) = 0;

UE_HSDPA_rxpower(1:MAX_NUM_BS,1:MAX_NUM_UE) = 0;
SINR_HSDPA(1:MAX_NUM_BS,1:MAX_NUM_UE) = 0;
SINR_HSDPA_dB(1:MAX_NUM_BS,1:MAX_NUM_UE) = 0;

UE_AP_rxpower(1:MAX_NUM_AP,1:MAX_NUM_UE) = 0;
SINR_AP(1:MAX_NUM_AP,1:MAX_NUM_UE) = 0;
SINR_AP_dB(1:MAX_NUM_AP,1:MAX_NUM_UE) = 0;

BS_BitRate_DL(1:MAX_NUM_ALL,1:END_TIME) = 0; %
BS_Throughput_DL(1:MAX_NUM_ALL,1:END_TIME) = 0; % BS throghput every second
BS_Utilization_DL(1:MAX_NUM_ALL,1:END_TIME) = 0;
BS_Throughput_DL_Avg(1:MAX_NUM_ALL) = 0;
BS_P_Blocking(1:MAX_NUM_ALL) = 0;

AP_txpower(1:MAX_NUM_AP) = 0.1;

UE_Throughput_DL(1:MAX_NUM_UE,1:END_TIME) = 0;  %the real traffic at each user, as BitRate is the attempted/request traffic by the UE
UE_Cost(1:MAX_NUM_UE,1:END_TIME) = 0; 

%handover thresholds initialization
Th_add(1:MAX_NUM_ALL)		= 0;	%add margin
Th_drop(1:MAX_NUM_ALL)		= 0;	%drop margin
Th_rep(1:MAX_NUM_ALL)		= 0;	%replacement Hysteresis

Th_add_UE(1:MAX_NUM_UE)		= 0;	%add margin
Th_drop_UE(1:MAX_NUM_UE)	= 0;	%drop margin
Th_rep_UE(1:MAX_NUM_UE)		= 0;	%replacement Hysteresis


ActiveSet(1:MAX_NUM_UE, 1:4) = 0;

Drop_Timer(1:MAX_NUM_UE,1:MAX_NUM_ALL)= 0;   %Time to trigger drop
Add_Timer(1:MAX_NUM_UE,1:MAX_NUM_ALL) = 0;   %Time to trigger add
Rep_Timer(1:MAX_NUM_UE,1:MAX_NUM_ALL) = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% start simulation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for t=1:END_TIME

%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Downlink inteference and load factor
    % Calculated from power measurements left from (t-1)
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %initialize downlink dedicated channel (DCH) tx power for each BS
    if t==1
        for UE=1:MAX_NUM_UE
            [i,j]=min(PathLoss(:,UE,t));
            ActiveSet(UE,1)= j;
            if (BitRate_DL(UE,1) == RATE_1) || (BitRate_DL(UE,1) == RATE_2)
                %BS_DCH_txpower(ActiveSet(UE,1),UE) = 0.1;
                UE_Throughput_DL(UE,1) = BitRate_DL(UE,1); %for t=1, throughput equal to bitrate
            end
        end
    end

    % Downlink Load Factor
    BS_txpower = BS_HSDPA_txpower + ALPHA*(BS_Pilot_txpower + BS_Common_txpower); %row vector(1:11), BS total tx power = sum(traffic channel power)+pilot channel power

    % Downlink total interferences
    for UE=1:MAX_NUM_UE
        UE_total_rxpower(UE) = sum(BS_txpower'./(10.^(PathLoss(:,UE,t)/10)));
        UE_total_rxpower_AP(UE) = sum(AP_txpower'./((10.^(PathLoss_AP(:,UE,t)/10))+Pn_AP));
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % cell selection and handover
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    for UE=1:MAX_NUM_UE
        UE_HSDPA_rxpower(:,UE) = BS_HSDPA_txpower'./(10.^(PathLoss(:,UE,t)/10));
        SINR_HSDPA(:,UE) = UE_HSDPA_rxpower(:,UE)/UE_total_rxpower(UE);
        SINR_HSDPA_dB(:,UE) = 10*log10(UE_HSDPA_rxpower(:,UE)/UE_total_rxpower(UE));

        UE_AP_rxpower(:,UE) = AP_txpower'./(10.^(PathLoss_AP(:,UE,t)/10));
        SINR_AP(:,UE) = func_sinr(UE_AP_rxpower(:,UE)/UE_total_rxpower_AP(UE));
        SINR_AP_dB(:,UE) = 10*log10(UE_AP_rxpower(:,UE)/UE_total_rxpower(UE));

        SINR_ALL = [SINR_HSDPA;SINR_AP];
        SINR_ALL_dB = [SINR_HSDPA_dB;SINR_AP_dB];
    end

    for UE=1:MAX_NUM_UE
        
%% Hard Handover

if Handover_Type == 1 %RSS, with pre-defined Th_RSS
    
    [Best_AP_RSS,Best_Cell]= max(UE_AP_rxpower(:,UE));
    if 10*log10(Best_AP_RSS*1000) > Th_RSS  %converte to dBm
        ActiveSet(UE,1) = Best_Cell + 7;
    else
        [Best_Cell_Ec_Io,Best_Cell] = max(SINR_HSDPA(:,UE));
        ActiveSet(UE,1) = Best_Cell;
    end
    
elseif Handover_Type == 2 %CSVH non_adaptive
    [Best_Cell_Ec_Io,Best_Cell] = max(SINR_ALL(:,UE));
    ActiveSet(UE,1) = Best_Cell;
    

elseif Handover_Type == 6 %MASVH adaptive_cost_utilization with b
    if BitRate_DL(UE,t) == 0
        [Best_Cell_Ec_Io,Best_Cell] = max(SINR_ALL(:,UE));
        ActiveSet(UE,1) = Best_Cell;
    else %input of BitRate cannot be Zero
        if t==1 %utilization not avaiable yet
            Best_Cell = func_cell_selection_by_cost(SINR_ALL(:,UE),BitRate_DL(UE,t));
        else
            Best_Cell = func_cell_selection_by_utilization_cost_b(SINR_ALL(:,UE),BitRate_DL(UE,t), mean(BS_Utilization_DL(:,1:(t-1))')',Parameter_B);
        end
        ActiveSet(UE,1) = Best_Cell;
    end
    
end
         
    end


    ActiveSet_ALL(:,:,t) = ActiveSet; %ActiveSet(UE,1:2,t)

%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Call Admission Control, 
    % 
    % 
    %														
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    for UE=1:MAX_NUM_UE
        if BitRate_DL(UE,t) ~= 0	%has traffic at t
            if BS_Utilization_DL(ActiveSet(UE,1),t) < 1   % not busy
                Max_Rate = func_rate(max(SINR_ALL(ActiveSet(UE,find(ActiveSet(UE,:)~=0)),UE)));
                if BitRate_DL(UE,t) < Max_Rate
                    UE_Throughput_DL(UE,t) = BitRate_DL(UE,t);
                else
                    UE_Throughput_DL(UE,t) = Max_Rate;
                end
                % update BS throughput and utilization
                for i=1:4 %active set of 4
                    if ActiveSet(UE,i)~=0
                        BS_Throughput_DL(ActiveSet(UE,i),t) = BS_Throughput_DL(ActiveSet(UE,i),t) + UE_Throughput_DL(UE,t);
                        BS_Utilization_DL(ActiveSet(UE,i),t) = func_utilization(ActiveSet(UE,i), BS_Throughput_DL(ActiveSet(UE,i),t));
                    end
                end
            else
                UE_Throughput_DL(UE,t) = 0;
            end
        else  %birrate=0, so no throughput
            UE_Throughput_DL(UE,t) = 0;
        end
    end

%%    

end

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate cell blocking and throughput
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% throughput of each cell for each t
for t=1:END_TIME
	for UE=1:MAX_NUM_UE
		for i=1:4 %active set of 4
			if ActiveSet_ALL(UE,i,t)~=0
				BS_BitRate_DL(ActiveSet_ALL(UE,i,t),t) = BS_BitRate_DL(ActiveSet_ALL(UE,i,t),t) + BitRate_DL(UE,t);               
                UE_Cost(UE,t) = func_cost(ActiveSet_ALL(UE,i,t));
			end
		end
	end
end

Avg_BS_Throughput_DL = mean(mean(BS_Throughput_DL(1:MAX_NUM_BS,:)'));
Avg_AP_Throughput_DL = mean(mean(BS_Throughput_DL((MAX_NUM_BS+1):MAX_NUM_ALL,:)'));
Avg_Sys_Throughput_DL = mean(mean(BS_Throughput_DL'));
Avg_Sys_Blocking_DL = (sum(sum(BS_BitRate_DL')) - sum(sum(BS_Throughput_DL')))/sum(sum(BS_BitRate_DL'));
Avg_UE_Cost = mean(sum((UE_Cost.*UE_Throughput_DL)')./(sum(UE_Throughput_DL')+1e-100));

Avg_BS_Utilization_DL = mean(mean(BS_Utilization_DL(1:MAX_NUM_BS,:)'));
Avg_AP_Utilization_DL = mean(mean(BS_Utilization_DL((MAX_NUM_BS+1):MAX_NUM_ALL,:)'));
Avg_Sys_Utilization_DL = mean(mean(BS_Utilization_DL'));
